//! Implements Avarice message: target service, message type, json parameters
use serde_json;
use serde_json::json;
use std::fmt;

pub struct AvariceMessage {
    pub service: String,
    pub message_type: String,
    pub parameters: serde_json::Value,
}

impl AvariceMessage {
    /// Parses JSON form of a message into `AvariceMessage` struct
    pub fn from(message_str: &str) -> Option<AvariceMessage> {
        let mut message_json: serde_json::Value = serde_json::from_str(message_str).unwrap();
        let message_json = message_json.as_object_mut()?;
        Some(AvariceMessage {
            service: message_json.remove("s")?.as_str()?.to_owned(),
            message_type: message_json.remove("t")?.as_str()?.to_owned(),
            parameters: message_json.remove("p")?,
        })
    }
}

impl fmt::Display for AvariceMessage {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(
            f,
            "{}",
            json!({
                "s": self.service,
                "t": self.message_type,
                "p": self.parameters,
            })
            .to_string()
        )
    }
}
